@@ -22,11 +22,16 @@ import static org.apache.dolphinscheduler.common.Constants.CMDPARAM_COMPLEMENT_D
 import static org.apache.dolphinscheduler.common.Constants.CMD_PARAM_RECOVERY_START_NODE_STRING;
 import static org.apache.dolphinscheduler.common.Constants.CMD_PARAM_START_NODES;
 import static org.apache.dolphinscheduler.common.Constants.DEFAULT_WORKER_GROUP;
+import static org.apache.dolphinscheduler.common.Constants.SEC_2_MINUTES_TIME_UNIT;
+import static org.apache.dolphinscheduler.common.Constants.START_UP_PARAMS_PREFIX;
+import static org.apache.dolphinscheduler.common.Constants.GLOBAL_PARAMS_PREFIX;
+import static org.apache.dolphinscheduler.common.enums.DataType.VARCHAR;
+import static org.apache.dolphinscheduler.common.enums.Direct.IN;
 
+import org.apache.commons.collections4.MapUtils;
 import org.apache.dolphinscheduler.common.Constants;
 import org.apache.dolphinscheduler.common.enums.CommandType;
 import org.apache.dolphinscheduler.common.enums.DependResult;
-import org.apache.dolphinscheduler.common.enums.Direct;
 import org.apache.dolphinscheduler.common.enums.ExecutionStatus;
 import org.apache.dolphinscheduler.common.enums.FailureStrategy;
 import org.apache.dolphinscheduler.common.enums.Flag;
@@ -41,6 +46,7 @@ import org.apache.dolphinscheduler.common.model.TaskNode;
 import org.apache.dolphinscheduler.common.model.TaskNodeRelation;
 import org.apache.dolphinscheduler.common.process.ProcessDag;
 import org.apache.dolphinscheduler.common.process.Property;
+import org.apache.dolphinscheduler.common.thread.ThreadUtils;
 import org.apache.dolphinscheduler.common.utils.DateUtils;
 import org.apache.dolphinscheduler.common.utils.JSONUtils;
 import org.apache.dolphinscheduler.common.utils.NetUtils;
@@ -78,6 +84,7 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -398,9 +405,14 @@ public class WorkflowExecuteThread implements Runnable {
             return true;
         }
         if (task.getState().typeIsFinished()) {
-            if (completeTaskList.containsKey(Long.toString(task.getTaskCode())) && completeTaskList.get(Long.toString(task.getTaskCode())).getId() == task.getId()) {
+            if (completeTaskList.containsKey(Long.toString(task.getTaskCode())) && completeTaskList.get(Long.toString(task.getTaskCode())).getId() == task.getId()
+                && task.getState() != ExecutionStatus.NEED_FAULT_TOLERANCE) {
                 return true;
             }
+            if (task.getStartTime() == null) {
+                logger.info("Maybe TASK_EXECUTE_ACK has not been received when the task finish, will wait for one second");
+                ThreadUtils.sleep(Constants.SLEEP_TIME_MILLIS);
+            }
             taskFinished(task);
             return true;
         }
@@ -420,8 +432,11 @@ public class WorkflowExecuteThread implements Runnable {
 
     private void taskFinished(TaskInstance task) {
         logger.info("work flow {} task {} state:{} ", processInstance.getId(), task.getId(), task.getState());
-        if (task.isDependTask() && task.getState() == ExecutionStatus.NEED_FAULT_TOLERANCE) {
-            logger.info("resubmit NEED_FAULT_TOLERANCE dependent task");
+        if (task.getState() == ExecutionStatus.NEED_FAULT_TOLERANCE) {
+            logger.info("resubmit NEED_FAULT_TOLERANCE {} task", task.getId());
+            if (task.getMaxRetryTimes() == 0) {
+                task.setRetryTimes(task.getRetryTimes() + 1);
+            }
             addTaskToStandByList(task);
             submitStandByTask();
             return;
@@ -461,7 +476,7 @@ public class WorkflowExecuteThread implements Runnable {
         taskRetryCheckList.remove(task.getId());
         depStateCheckList.remove(task.getId());
         if (task.getState().typeIsSuccess()) {
-            processInstance.setVarPool(task.getVarPool());
+            // processInstance.setVarPool(task.getVarPool());
             processService.saveProcessInstance(processInstance);
             submitPostNode(Long.toString(task.getTaskCode()));
         } else if (task.getState().typeIsFailure()) {
@@ -492,7 +507,7 @@ public class WorkflowExecuteThread implements Runnable {
             logger.info("process:{} state {} change to {}", processInstance.getId(), processInstance.getState(), stateEvent.getExecutionStatus());
             processInstance = processService.findProcessInstanceById(this.processInstance.getId());
 
-            if (stateEvent.getExecutionStatus() == ExecutionStatus.STOP) {
+            if (stateEvent.getExecutionStatus().typeIsCancel()) {
                 this.updateProcessInstanceState(stateEvent);
                 return true;
             }
@@ -520,7 +535,7 @@ public class WorkflowExecuteThread implements Runnable {
             return false;
         }
 
-        if (processInstance.getState() == ExecutionStatus.READY_STOP) {
+        if (processInstance.getState().typeIsReadyCancel()) {
             return false;
         }
 
@@ -529,14 +544,18 @@ public class WorkflowExecuteThread implements Runnable {
             scheduleDate = complementListDate.get(0);
         } else if (processInstance.getState().typeIsFinished()) {
             endProcess();
+            // rerun process instance of complement didn't need create the next process complement
+            if (processInstance.getCommandType() == CommandType.REPEAT_RUNNING) {
+                return true;
+            }
             if (complementListDate.size() <= 0) {
                 logger.info("process complement end. process id:{}", processInstance.getId());
                 return true;
             }
             int index = complementListDate.indexOf(scheduleDate);
-            if (index >= complementListDate.size() - 1 || !processInstance.getState().typeIsSuccess()) {
+            if (index >= complementListDate.size() - 1) {
                 logger.info("process complement end. process id:{}", processInstance.getId());
-                // complement data ends || no success
+                // complement data ends
                 return true;
             }
             logger.info("process complement continue. process id:{}, schedule time:{} complementListDate:{}",
@@ -679,6 +698,9 @@ public class WorkflowExecuteThread implements Runnable {
         if (processInstance.isComplementData() && complementListDate.size() == 0) {
             Map<String, String> cmdParam = JSONUtils.toMap(processInstance.getCommandParam());
             if (cmdParam != null && cmdParam.containsKey(CMDPARAM_COMPLEMENT_DATA_START_DATE)) {
+                // reset global params while there are start parameters
+                setGlobalParamIfCommanded(processDefinition, cmdParam);
+
                 Date start = DateUtils.stringToDate(cmdParam.get(CMDPARAM_COMPLEMENT_DATA_START_DATE));
                 Date end = DateUtils.stringToDate(cmdParam.get(CMDPARAM_COMPLEMENT_DATA_END_DATE));
                 List<Schedule> schedules = processService.queryReleaseSchedulerListByProcessDefinitionCode(processInstance.getProcessDefinitionCode());
@@ -925,16 +947,12 @@ public class WorkflowExecuteThread implements Runnable {
             if (allProperty.size() > 0) {
                 taskInstance.setVarPool(JSONUtils.toJsonString(allProperty.values()));
             }
-        } else {
-            if (StringUtils.isNotEmpty(processInstance.getVarPool())) {
-                taskInstance.setVarPool(processInstance.getVarPool());
-            }
         }
     }
 
     private void setVarPoolValue(Map<String, Property> allProperty, Map<String, TaskInstance> allTaskInstance, TaskInstance preTaskInstance, Property thisProperty) {
         //for this taskInstance all the param in this part is IN.
-        thisProperty.setDirect(Direct.IN);
+        thisProperty.setDirect(IN);
         //get the pre taskInstance Property's name
         String proName = thisProperty.getProp();
         //if the Previous nodes have the Property of same name
@@ -972,7 +990,12 @@ public class WorkflowExecuteThread implements Runnable {
                 continue;
             }
             TaskInstance task = createTaskInstance(processInstance, taskNodeObject);
-            taskInstances.add(task);
+            if (processInstance.getState().typeIsReadyPause() && (!activeTaskProcessorMaps.isEmpty() || !completeTaskList.isEmpty())) {
+                task.setState(ExecutionStatus.PAUSE);
+                completeTaskList.put(String.valueOf(task.getTaskCode()), task);
+            } else {
+                taskInstances.add(task);
+            }
         }
 
         // if previous node success , post node submit
@@ -986,7 +1009,7 @@ public class WorkflowExecuteThread implements Runnable {
                 logger.info("task {} has already run success, task id:{}", task.getName(), task.getId());
                 continue;
             }
-            if (task.getState().typeIsPause() || task.getState().typeIsCancel()) {
+            if (task.getState().typeIsCancel()) {
                 logger.info("task {} stopped, the state is {}, task id:{}", task.getName(), task.getState(), task.getId());
             } else {
                 addTaskToStandByList(task);
@@ -1018,7 +1041,7 @@ public class WorkflowExecuteThread implements Runnable {
                     return DependResult.WAITING;
                 }
                 ExecutionStatus depTaskState = completeTaskList.get(depsNode).getState();
-                if (depTaskState.typeIsPause() || depTaskState.typeIsCancel()) {
+                if (depTaskState.typeIsCancel()) {
                     return DependResult.NON_EXEC;
                 }
                 // ignore task state if current task is condition
@@ -1128,16 +1151,9 @@ public class WorkflowExecuteThread implements Runnable {
         } else {
             if (processInstance.getCommandType() == CommandType.RECOVER_TOLERANCE_FAULT_PROCESS
                 || processInstance.getCommandType() == CommandType.RECOVER_SUSPENDED_PROCESS) {
-                List<Integer> failedList = processService.findTaskIdByInstanceState(processInstance.getId(), ExecutionStatus.FAILURE);
-                if (!failedList.isEmpty()) {
-                    return true;
-                }
-                List<Integer> toleranceList = processService.findTaskIdByInstanceState(processInstance.getId(), ExecutionStatus.NEED_FAULT_TOLERANCE);
-                if (!toleranceList.isEmpty()) {
-                    return true;
-                }
-                List<Integer> killedList = processService.findTaskIdByInstanceState(processInstance.getId(), ExecutionStatus.KILL);
-                if (!killedList.isEmpty()) {
+                List<Integer> failureTaskIds = processService.findLastTaskIdByStateList(processInstance.getId(),
+                    Lists.newArrayList(ExecutionStatus.FAILURE, ExecutionStatus.NEED_FAULT_TOLERANCE, ExecutionStatus.KILL));
+                if (!failureTaskIds.isEmpty()) {
                     return true;
                 }
             }
@@ -1186,9 +1202,7 @@ public class WorkflowExecuteThread implements Runnable {
         }
 
         List<TaskInstance> pauseList = getCompleteTaskByState(ExecutionStatus.PAUSE);
-        if (CollectionUtils.isNotEmpty(pauseList)
-                || !isComplementEnd()
-                || readyToSubmitTaskQueue.size() > 0) {
+        if (CollectionUtils.isNotEmpty(pauseList) || !isComplementEnd()) {
             return ExecutionStatus.PAUSE;
         } else {
             return ExecutionStatus.SUCCESS;
@@ -1440,12 +1454,26 @@ public class WorkflowExecuteThread implements Runnable {
         return this.processInstance.getState().typeIsFinished();
     }
 
+    public boolean activeTaskFinish() {
+        if (activeTaskProcessorMaps.isEmpty()) {
+            return true;
+        }
+        List<TaskInstance> taskInstanceList = processService.findTaskInstanceListByIds(activeTaskProcessorMaps.keySet());
+        for (TaskInstance taskInstance : taskInstanceList) {
+            if (!taskInstance.getState().typeIsFinished()) {
+                return false;
+            }
+        }
+        return true;
+    }
+
     /**
      * handling the list of tasks to be submitted
      */
     private void submitStandByTask() {
         try {
             int length = readyToSubmitTaskQueue.size();
+            List<TaskInstance> skipSubmitInstances = new ArrayList<>();
             for (int i = 0; i < length; i++) {
                 TaskInstance task = readyToSubmitTaskQueue.peek();
                 if (task == null) {
@@ -1462,11 +1490,27 @@ public class WorkflowExecuteThread implements Runnable {
                         submitPostNode(Long.toString(task.getTaskCode()));
                         continue;
                     }
+                    if (retryTask != null && retryTask.getState() == ExecutionStatus.FAILURE && retryTask.getMaxRetryTimes() !=0 && retryTask.getRetryInterval() != 0) {
+                        long failedTimeInterval = DateUtils.differSec(new Date(), retryTask.getEndTime());
+                        if ((long) retryTask.getRetryInterval() * SEC_2_MINUTES_TIME_UNIT > failedTimeInterval) {
+                            logger.info("task name: {} retry waiting has not exceeded the interval time, and skip submission this time, task id:{}", task.getName(), task.getId());
+                            readyToSubmitTaskQueue.remove(task);
+                            skipSubmitInstances.add(task);
+                            continue;
+                        }
+                    }
                 }
                 //init varPool only this task is the first time running
                 if (task.isFirstRun()) {
                     //get pre task ,get all the task varPool to this task
-                    Set<String> preTask = dag.getPreviousNodes(Long.toString(task.getTaskCode()));
+                    Set<String> preTask = new HashSet<>();
+                    preTask.addAll(dag.getPreviousNodes(Long.toString(task.getTaskCode())));
+                    TaskNode taskNode = dag.getNode(Long.toString(task.getTaskCode()));
+                    if (null != taskNode && null != taskNode.getDepList() && !taskNode.getDepList().isEmpty()) {
+                        logger.debug("in submitStandByTask: taskCode:{}, taskType: {}, preTasks: {}, depList:{}",
+                                task.getTaskCode(), taskNode.getType(), taskNode.getPreTasks(), taskNode.getDepList());
+                        preTask.addAll(taskNode.getDepList());
+                    }
                     getPreVarPool(task, preTask);
                 }
                 DependResult dependResult = getDependResultForTask(task);
@@ -1492,6 +1536,10 @@ public class WorkflowExecuteThread implements Runnable {
                     logger.info("remove task {},id:{} , because depend result : {}", task.getName(), task.getId(), dependResult);
                 }
             }
+            for (TaskInstance task : skipSubmitInstances) {
+                readyToSubmitTaskQueue.put(task);
+            }
+            skipSubmitInstances.clear();
         } catch (Exception e) {
             logger.error("submit standby task error", e);
         }
@@ -1598,4 +1646,36 @@ public class WorkflowExecuteThread implements Runnable {
     public Map<Integer, ITaskProcessor> getActiveTaskProcessorMaps() {
         return activeTaskProcessorMaps;
     }
+
+    private void setGlobalParamIfCommanded(ProcessDefinition processDefinition, Map<String, String> cmdParam) {
+        // get start params from command param
+        Map<String, String> startParamMap = new HashMap<>();
+        if (cmdParam.containsKey(Constants.CMD_PARAM_START_PARAMS)) {
+            String startParamJson = cmdParam.get(Constants.CMD_PARAM_START_PARAMS);
+            startParamMap = JSONUtils.toMap(startParamJson);
+        }
+        Map<String, String> fatherParamMap = new HashMap<>();
+        if (cmdParam.containsKey(Constants.CMD_PARAM_FATHER_PARAMS)) {
+            String fatherParamJson = cmdParam.get(Constants.CMD_PARAM_FATHER_PARAMS);
+            fatherParamMap = JSONUtils.toMap(fatherParamJson);
+        }
+        startParamMap.putAll(fatherParamMap);
+        Map<String, String> globalMap = processDefinition.getGlobalParamMap();
+        List<Property> globalParamList = processDefinition.getGlobalParamList();
+        if (MapUtils.isNotEmpty(startParamMap) && globalMap != null) {
+            Map<String, String> tempGlobalMap = new HashMap<>();
+            // add prefix for global params
+            for (Map.Entry<String, String> param : globalMap.entrySet()) {
+                tempGlobalMap.put(GLOBAL_PARAMS_PREFIX+ param.getKey(), param.getValue());
+            }
+            globalParamList.forEach(property -> property.setProp(GLOBAL_PARAMS_PREFIX + property.getProp()));
+            // set start param into global params, add prefix for startup params
+            for (Entry<String, String> startParam : startParamMap.entrySet()) {
+                String tmpStartParamKey = START_UP_PARAMS_PREFIX + startParam.getKey();
+                tempGlobalMap.put(tmpStartParamKey, startParam.getValue());
+                globalParamList.add(new Property(tmpStartParamKey, IN, VARCHAR, startParam.getValue()));
+            }
+            processDefinition.setGlobalParamMap(tempGlobalMap);
+        }
+    }
 }
